@@ -14,6 +14,8 @@
 #' @importFrom graphics text
 #' @importFrom igraph all_simple_paths
 #' @importFrom igraph as_ids
+#' @importFrom igraph ends
+#' @importFrom igraph is_connected
 #' @importFrom igraph components
 #' @importFrom igraph degree
 #' @importFrom igraph graph_from_data_frame
@@ -33,3 +35,24 @@
 NULL
 
 .datatable.aware <- TRUE
+
+rTwigStartupMessage <- function() {
+  msg <- c(
+    paste0(
+      "rTwig version ",
+      utils::packageVersion("rTwig")
+    ),
+    "\nType 'citation(\"rTwig\")' for citing this R package in publications."
+  )
+  return(msg)
+}
+
+.onAttach <- function(lib, pkg) {
+  # startup message
+  msg <- rTwigStartupMessage()
+  if (!interactive()) {
+    msg[1] <- paste("Package 'rTwig' version", utils::packageVersion("rTwig"))
+  }
+  packageStartupMessage(msg)
+  invisible()
+}
